@@ -3,13 +3,13 @@
 
    Model File:  perc.model
 
-   Date:  Mon May  9 16:22:27 2016
+   Date:  Fri Feb 23 09:33:16 2018
 
-   Created by:  "mod v5.6.5"
+   Created by:  "../../mod/mod v6.0.0"
     -- a model preprocessor by Don Maszle
    ___________________________________________________
 
-   Copyright (c) 1993-2015 Free Software Foundation, Inc.
+   Copyright (c) 1993-2018 Free Software Foundation, Inc.
 
    Model calculations for compartmental model:
 
@@ -147,7 +147,7 @@
 /* For export. Keep track of who we are. */
 char szModelDescFilename[] = "perc.model";
 char szModelSourceFilename[] = __FILE__;
-char szModelGenAndVersion[] = "mod v5.6.5";
+char szModelGenAndVersion[] = "../../mod/mod v6.0.0";
 
 /* Externs */
 extern BOOL vbModelReinitd;
