@@ -1,6 +1,6 @@
 /* mod.c
 
-   Copyright (c) 1993-2012. Free Software Foundation, Inc.
+   Copyright (c) 1993-2017. Free Software Foundation, Inc.
 
    This file is part of GNU MCSim.
 
@@ -17,14 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with GNU MCSim; if not, see <http://www.gnu.org/licenses/>
 
-   -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
-      Author:  @a
-   -- SCCS  ---------
-
    Entry point for model code generator.
 
    Calls ReadModel() to define the model and WriteModel() to create
@@ -276,8 +268,9 @@ void GetCmdLineArgs (int nArg, char *const *rgszArg, PSTR *pszFileIn,
 void InitInfo (PINPUTINFO pinfo, PSTR szModGenName)
 {
   pinfo->wContext       = CN_GLOBAL;
-  pinfo->bTemplateInUse = FALSE;
+  pinfo->bDelays        = FALSE;
   pinfo->bforR          = FALSE;
+  pinfo->bTemplateInUse = FALSE;
   pinfo->szModGenName   = szModGenName;
 
   pinfo->pvmGloVars     = NULL;
@@ -294,6 +287,91 @@ void InitInfo (PINPUTINFO pinfo, PSTR szModGenName)
 } /* InitInfo */
 
 
+/* ----------------------------------------------------------------------------
+   Cleanup
+
+   deallocate memory.
+*/
+
+void Cleanup (PINPUTINFO pinfo)
+{
+  PVMMAPSTRCT  next;
+
+  while (pinfo->pvmGloVars) {
+    next = pinfo->pvmGloVars->pvmNextVar;
+    free(pinfo->pvmGloVars->szName);
+    free(pinfo->pvmGloVars->szEqn);
+    free(pinfo->pvmGloVars);
+    pinfo->pvmGloVars = next;
+  }
+
+  while (pinfo->pvmDynEqns) {
+    next = pinfo->pvmDynEqns->pvmNextVar;
+    free(pinfo->pvmDynEqns->szName);
+    free(pinfo->pvmDynEqns->szEqn);
+    free(pinfo->pvmDynEqns);
+    pinfo->pvmDynEqns = next;
+  }
+  
+  while (pinfo->pvmScaleEqns) {
+    next = pinfo->pvmScaleEqns->pvmNextVar;
+    free(pinfo->pvmScaleEqns->szName);
+    free(pinfo->pvmScaleEqns->szEqn);
+    free(pinfo->pvmScaleEqns);
+    pinfo->pvmScaleEqns = next;
+  }
+  
+  while (pinfo->pvmJacobEqns) {
+    next = pinfo->pvmJacobEqns->pvmNextVar;
+    free(pinfo->pvmJacobEqns->szName);
+    free(pinfo->pvmJacobEqns->szEqn);
+    free(pinfo->pvmJacobEqns);
+    pinfo->pvmJacobEqns = next;
+  }
+
+  while (pinfo->pvmCalcOutEqns) {
+    next = pinfo->pvmCalcOutEqns->pvmNextVar;
+    free(pinfo->pvmCalcOutEqns->szName);
+    free(pinfo->pvmCalcOutEqns->szEqn);
+    free(pinfo->pvmCalcOutEqns);
+    pinfo->pvmCalcOutEqns = next;
+  }
+
+  while (pinfo->pvmEventEqns) {
+    next = pinfo->pvmEventEqns->pvmNextVar;
+    free(pinfo->pvmEventEqns->szName);
+    free(pinfo->pvmEventEqns->szEqn);
+    free(pinfo->pvmEventEqns);
+    pinfo->pvmEventEqns = next;
+  }
+
+  while (pinfo->pvmRootEqns) {
+    next = pinfo->pvmRootEqns->pvmNextVar;
+    free(pinfo->pvmRootEqns->szName);
+    free(pinfo->pvmRootEqns->szEqn);
+    free(pinfo->pvmRootEqns);
+    pinfo->pvmRootEqns = next;
+  }
+
+  while (pinfo->pvmCpts) {
+    next = pinfo->pvmCpts->pvmNextVar;
+    free(pinfo->pvmCpts->szName);
+    free(pinfo->pvmCpts->szEqn);
+    free(pinfo->pvmCpts);
+    pinfo->pvmCpts = next;
+  }
+
+  while (pinfo->pvmLocalCpts) {
+    next = pinfo->pvmJacobEqns->pvmNextVar;
+    free(pinfo->pvmJacobEqns->szName);
+    free(pinfo->pvmJacobEqns->szEqn);
+    free(pinfo->pvmJacobEqns);
+    pinfo->pvmJacobEqns = next;
+  }
+
+} /* Cleanup */
+
+
 /* ----------------------------------------------------------------------------
    main -- Entry point for the simulation model preprocessor
    
@@ -307,15 +385,6 @@ int main (int nArg, PSTR rgszArg[])
 
   AnnounceProgram ();
 
-#ifdef WIN32
-  /* replace '\' in rgszArg[0] with '/' */
-  { int i;
-    for (i = 0; i < strlen(rgszArg[0]); i++) {
-      if (rgszArg[0][i] == '\\') rgszArg[0][i] = '/';
-    }
-  }
-#endif
-
   InitInfo (&info, rgszArg[0]);
   InitInfo (&tempinfo, rgszArg[0]);
 
@@ -332,6 +401,8 @@ int main (int nArg, PSTR rgszArg[])
   else 
     WriteModel (&info, szFileOut);
 
+  Cleanup (&info);
+
   return 0;
 
 } /* main */
