@@ -3,7 +3,7 @@
    Written by Frederic Bois
    5 January 1996
 
-   Copyright (c) 1996-2008 Free Software Foundation, Inc.
+   Copyright (c) 1996-2017 Free Software Foundation, Inc.
 
    This file is part of GNU MCSim.
 
@@ -20,14 +20,6 @@
    You should have received a copy of the GNU General Public License
    along with GNU MCSim; if not, see <http://www.gnu.org/licenses/>
 
-   -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
-      Author:  @a
-   -- SCCS  ---------
-
    This file calls at least one GNU Scientific Library function, if the
    symbol HAVE_LIBGSL is defined in config.h (or in the makefile).
    Otherwise, the corresponding features are disabled (and the program
@@ -53,7 +45,6 @@
 
    Print out the type of simulations to be performed.
 */
-
 void AnnounceMarkov (int nSimTypeFlag, long nIter)
 {
 
@@ -347,10 +338,10 @@ void CloneMCVarsL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
   ++pMCVar->iDepth;
   for (i = 0; i < plevel->iInstances; i++) {
     pLower = plevel->pLevels[i];
-    if (!(pClone = (PMCVAR) malloc (sizeof (MCVAR))))
-      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "CloneMCVarsL", NULL);
+    if (!(pClone = (PMCVAR) malloc(sizeof (MCVAR))))
+      ReportError(NULL, RE_OUTOFMEM | RE_FATAL, "CloneMCVarsL", NULL);
     memcpy (pClone, pMCVar, sizeof (MCVAR));
-    pClone->plistDependents = InitList ();
+    pClone->plistDependents = InitList();
     pLower->rgpMCVars[*pnMCVars] = pClone;
   }
   ++(*pnMCVars);
@@ -358,7 +349,6 @@ void CloneMCVarsL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 } /* CloneMCVarsL */
 
 
-
 /* Function -------------------------------------------------------------------
    CloseMarkovFiles
 
@@ -366,7 +356,6 @@ void CloneMCVarsL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
    The restart file has already been closed by the ReadRestart
 
 */
-
 void CloseMarkovFiles (PANALYSIS panal)
 {
   if (panal->gd.pfileOut) {
@@ -383,7 +372,6 @@ void CloseMarkovFiles (PANALYSIS panal)
    Converts lists to arrays
    Called from TraverseLevels
 */
-
 void ConvertLists(PLEVEL plevel, char **args)
 {
   PANALYSIS panal = (PANALYSIS)args[0];
@@ -428,7 +416,6 @@ void ConvertLists(PLEVEL plevel, char **args)
    Set default values for inverse temperatures of tempered MCMC algorithms, if
    the user has not specified values.
 */
-
 void SetInvTemperatures (PGIBBSDATA pgd) {
 
   /* inverse temperatures not set by the user: use the default */
@@ -448,6 +435,10 @@ void SetInvTemperatures (PGIBBSDATA pgd) {
         !(pgd->rglTemp = InitlVector (NTEMP)))
       ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "SetInvTemperatures", NULL);
 
+    /* initialize rglTemp, counting perk visits */
+    for (i = 0; i < NTEMP; i++)
+      pgd->rglTemp[i] = 0;
+
     /* initialize the inverse temperatures */
     pgd->rgInvTemperatures[0] = 0.4;                         /* hot */
 
@@ -466,7 +457,6 @@ void SetInvTemperatures (PGIBBSDATA pgd) {
 
    Core routine of the MCMC sampler
 */
-
 void DoMarkov (PANALYSIS panal)
 {
   PGIBBSDATA pgd = &panal->gd;
@@ -483,11 +473,11 @@ void DoMarkov (PANALYSIS panal)
 
   AnnounceMarkov (pgd->nSimTypeFlag, nIter);
 
-  SetInvTemperatures (pgd); // (if needed)
+  SetInvTemperatures (pgd); /* (if needed) */
 
   OpenMarkovFiles (panal);
 
-  ReadDataFile (panal);     // (if needed)
+  ReadDataFile (panal);     /* (if needed) */
 
   /* MC variables must be placed in arrays at the next lower level */
   TraverseLevels (pLevel0, CloneMCVars, NULL);
@@ -785,7 +775,12 @@ void DoMarkov (PANALYSIS panal)
   CloseMarkovFiles (panal);
 
   if ((pgd->nSimTypeFlag == 3) || (pgd->nSimTypeFlag == 4)) {
-    printf ("\nTemperature count:\n");
+    printf ("\nInverse temperatures:\n");
+    for (i = 0; i < pgd->nInvTemperatures - 1; i++)
+      printf ("%f\t", pgd->rgInvTemperatures[i]);
+    printf ("%f\n", pgd->rgInvTemperatures[pgd->nInvTemperatures - 1]);
+
+    printf ("\nInverse temperatures' count:\n");
     for (i = 0; i < pgd->nInvTemperatures - 1; i++)
       printf ("%ld\t", pgd->rglTemp[i]);
     printf ("%ld\n\n", pgd->rglTemp[pgd->nInvTemperatures - 1]);
@@ -869,7 +864,6 @@ void FindLikeParents (PLEVEL plevel, char **args)
    Find the direct dependents of the MC vars at this level by looking at this
    and all lower levels
 */
-
 void FindMCDependents (PLEVEL plevel, char **args)
 {
   long   i, j;
@@ -974,7 +968,6 @@ void GetNumberOfMCVars (PLEVEL plevel, char **args)
    Sample initial values of thetas if not fixed
    Called from TraverseLevels
 */
-
 void InitMCVars(PLEVEL plevel, char **args)
 {
   FILE *pOutFile = (FILE*)args[0];
@@ -993,7 +986,6 @@ void InitMCVars(PLEVEL plevel, char **args)
 /* Function -------------------------------------------------------------------
    ListToPMCArray
 */
-
 void ListToPMCArray (PANALYSIS panal, PLIST plist,
                      long *pnMCVars, PMCVAR **rgpMCVars)
 {
@@ -1012,7 +1004,6 @@ void ListToPMCArray (PANALYSIS panal, PLIST plist,
 /* Function -------------------------------------------------------------------
    ListToPMCArrayL
 */
-
 void ListToPMCArrayL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 {
   PMCVAR pMCVar = (PMCVAR)pData;
@@ -1027,7 +1018,6 @@ void ListToPMCArrayL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 /* Function -------------------------------------------------------------------
    ListToPVArray
 */
-
 void ListToPVArray (PANALYSIS panal, PLIST plist,
                     long *pnFixedVars, PVARMOD **rgpFixedVars)
 {
@@ -1046,7 +1036,6 @@ void ListToPVArray (PANALYSIS panal, PLIST plist,
 /* Function -------------------------------------------------------------------
    ListToPVArrayL
 */
-
 void ListToPVArrayL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 {
   PVARMOD pVar = (PVARMOD)pData;
@@ -1116,7 +1105,12 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
                             pMCVar->pszName, "LnDensity");
       return -log(dTheta * (dParm2 - dParm1));
 
-    case MCV_NORMALV: dParm2 = sqrt (dParm2); /* fall thru */
+    case MCV_NORMALV:  dParm2 = sqrt (dParm2);
+      return lnDFNormal (dTheta, dParm1, dParm2);
+
+    case MCV_NORMALCV: dParm2 = fabs(dParm1 * dParm2);
+      return lnDFNormal (dTheta, dParm1, dParm2);
+ 
     case MCV_NORMAL:
     case MCV_HALFNORMAL:
       return lnDFNormal (dTheta, dParm1, dParm2);
@@ -1130,7 +1124,16 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
       }
       return (lnDFNormal(log(dTheta),log(dParm1),log(dParm2)) - log(dTheta));
 
-    case MCV_TRUNCNORMALV: dParm2 = sqrt (dParm2); /* fall thru */
+    case MCV_TRUNCNORMALV: dParm2 = sqrt (dParm2);
+      return lnDFNormal (dTheta, dParm1, dParm2) -
+             log(CDFNormal ((dMax - dParm1) / dParm2) -
+                 CDFNormal ((dMin - dParm1) / dParm2));
+
+    case MCV_TRUNCNORMALCV: dParm2 = fabs(dParm1 * dParm2);
+      return lnDFNormal (dTheta, dParm1, dParm2) -
+             log(CDFNormal ((dMax - dParm1) / dParm2) -
+                 CDFNormal ((dMin - dParm1) / dParm2));
+
     case MCV_TRUNCNORMAL:
       if (dParm2 <= 0.0) {
         sprintf(str, "%5.2e", dParm2);
@@ -1300,7 +1303,7 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
       /* Normal log-density minus log-jacobian */
       return lnDFNormal( dTmp4, dTmp3, dTmp ) - 0.5*log(pow(dTmp4,2) + dTmp2);
 
-  case MCV_STUDENTT: // Student t, dParm1 is dof, dParm2 is m, dMin is sigma
+  case MCV_STUDENTT: /* Student t, dParm1 is dof, dParm2 is m, dMin is sigma */
       if (dParm1 <= 0) {
         printf ("Error: bad dof for Student-T variate"
                 "in LnDensity\n");
@@ -1334,7 +1337,6 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
    returns the log-likelihood of the dependents given the parameter specified
    by pMCVar
 */
-
 double LnLike (PMCVAR pMCVar, PANALYSIS panal)
 {
   long n;
@@ -1357,7 +1359,6 @@ double LnLike (PMCVAR pMCVar, PANALYSIS panal)
 
    Likelihood of the data for one experiment
 */
-
 double LnLikeData (PLEVEL plevel, PANALYSIS panal) {
 
   PMCVAR pMCVar;
@@ -1435,7 +1436,6 @@ double LnLikeData (PLEVEL plevel, PANALYSIS panal) {
 
    get the maximum of MC variables
 */
-
 double MaxMCVar (PMCVAR pMCVar) {
 
   /* if the parameter has a discrete distribution, round it - FB 12/06/97 */
@@ -1459,7 +1459,6 @@ double MaxMCVar (PMCVAR pMCVar) {
 
     get the minimum of MC variables
 */
-
 double MinMCVar (PMCVAR pMCVar) {
 
   /* if the parameter has a discrete distribution, round it - FB 12/06/97 */
@@ -1483,14 +1482,16 @@ double MinMCVar (PMCVAR pMCVar) {
 
    Opens the output file and the restart file.
 */
-
 void OpenMarkovFiles (PANALYSIS panal)
 {
   /* Take care of the output file first */
 
   /* Use command line spec if given */
-  if (panal->bCommandLineSpec)
+  if (panal->bCommandLineSpec) {
+    free (panal->gd.szGout);
+    panal->bAllocatedFileName = FALSE;
     panal->gd.szGout = panal->szOutfilename;
+  }
 
   /* Default if none given */
   else if (!(panal->gd.szGout))
@@ -1516,7 +1517,6 @@ void OpenMarkovFiles (PANALYSIS panal)
 
    Run and print all experiments at level below `plevel'
 */
-
 void PrintAllExpts (PLEVEL plevel, PANALYSIS panal, PFILE pOutFile)
 {
   long n;
@@ -1535,7 +1535,6 @@ void PrintAllExpts (PLEVEL plevel, PANALYSIS panal, PFILE pOutFile)
 
    Called from TraverseLevels1
 */
-
 int PrintExpt (PLEVEL plevel, char **args)
 {
   PANALYSIS   panal = (PANALYSIS)args[0];
@@ -1617,7 +1616,6 @@ int PrintExpt (PLEVEL plevel, char **args)
    Upgraded by FB 27/03/99 to deal with improved data handling.
    Called from TraverseLevels.
 */
-
 void ReadData (PLEVEL plevel, char **args)
 {
   FILE *pfileData = (FILE *) args[0];
@@ -1670,7 +1668,6 @@ void ReadData (PLEVEL plevel, char **args)
 
    Reads in data from a specified data file, if specified.
 */
-
 void ReadDataFile (PANALYSIS panal)
 {
   if (panal->gd.szGdata) {
@@ -1702,7 +1699,6 @@ void ReadDataFile (PANALYSIS panal)
 
    For debugging, print the variables, parents, and dependencies
 */
-
 void PrintDeps (PLEVEL plevel, char **args)
 {
   long n, m;
@@ -1740,7 +1736,6 @@ void PrintDeps (PLEVEL plevel, char **args)
 
    initialize the parameters by reading them in the restart file.
 */
-
 void ReadRestart (FILE *pfileRestart, long nThetas,
                   double *pdTheta, double *pdSum, double **prgdSumProd,
                   long *pIter)
@@ -1805,8 +1800,7 @@ void ReadRestart (FILE *pfileRestart, long nThetas,
 
    initialize parameters and temperature stuff (pseudoprior, indexT) by reading
    them in the restart file.
-   */
-
+*/
 void ReadRestartTemper (FILE *pfileRestart, long nThetas, int nInvTemperatures,
                         double *pdTheta, double *pdSum, double **prgdSumProd,
                         long *pIter, long *pindexT, double *pdlnPi)
@@ -1889,7 +1883,6 @@ void ReadRestartTemper (FILE *pfileRestart, long nThetas, int nInvTemperatures,
 
    Called from TraverseLevels1
 */
-
 int RestoreLikelihoods (PLEVEL plevel, char **args)
 {
   PEXPERIMENT pExpt = plevel->pexpt;
@@ -1909,7 +1902,6 @@ int RestoreLikelihoods (PLEVEL plevel, char **args)
    Run all experiments (i.e. experiments at all levels below
    panal->pLevels[0]).
 */
-
 int RunAllExpts (PANALYSIS panal, PDOUBLE pdLnData)
 {
   PLEVEL plevel0 = panal->pLevels[0];
@@ -1939,7 +1931,6 @@ int RunAllExpts (PANALYSIS panal, PDOUBLE pdLnData)
 
    Called from TraverseLevels1
 */
-
 int RunExpt (PLEVEL plevel, char **args)
 {
   PANALYSIS   panal = (PANALYSIS) args[0];
@@ -1978,7 +1969,6 @@ int RunExpt (PLEVEL plevel, char **args)
 /* Function -------------------------------------------------------------------
    SampleTemperature
 */
-
 long SampleTemperature (PGIBBSDATA pgd, double dLnPrior, double dLnData)
 {
   int    indexT = pgd->indexT;
@@ -2049,7 +2039,6 @@ double SampleThetaUnif (PMCVAR pMCVar) {
 
    Called from TraverseLevels
 */
-
 void SampleThetas (PLEVEL plevel, char **args)
 {
   PANALYSIS  panal = (PANALYSIS) args[0];
@@ -2077,7 +2066,7 @@ void SampleThetas (PLEVEL plevel, char **args)
     if (pMCVar->bIsFixed)
       goto WriteIt;
 
-    // Compute prior and likelihood
+    /* Compute prior and likelihood */
     dLnPrior = LnDensity (pMCVar, panal);
     dLnLike  = LnLike (pMCVar, panal);
 
@@ -2153,9 +2142,9 @@ void SampleThetas (PLEVEL plevel, char **args)
 
       pMCVar->lJumps = 0; /* reset the jumps counter */
 
-    } // end of kernel stuff
+    } /* end of kernel stuff */
 
-    // compute the integral of the jump kernel
+    /* compute the integral of the jump kernel */
     dLnKern  = log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) /
                               pMCVar->dKernelSD) -
                    CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) /
@@ -2164,13 +2153,13 @@ void SampleThetas (PLEVEL plevel, char **args)
     /* sample a new value */
     pMCVar->dVal = SampleTheta (pMCVar);
 
-    // update the integral of the jump kernel
+    /* update the integral of the jump kernel */
     dLnKernNew  =  log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) /
                                   pMCVar->dKernelSD) -
                        CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) /
                                   pMCVar->dKernelSD));
 
-    // update prior and likelihood
+    /* update prior and likelihood */
     dLnPriorNew = LnDensity (pMCVar, panal);
     dLnLikeNew  = LnLike (pMCVar, panal);
 
@@ -2225,7 +2214,6 @@ void SampleThetas (PLEVEL plevel, char **args)
    Called from TraverseLevels
 
 */
-
 void SampleThetasTempered (PLEVEL plevel, char **args)
 {
   PANALYSIS  panal = (PANALYSIS) args[0];
@@ -2330,11 +2318,11 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
 
       pMCVar->lJumps = 0; /* reset the jumps counter */
 
-    } // end kernel updating stuff
+    } /* end kernel updating stuff */
 
     /* sample a new value */
 
-    if (pgd->rgInvTemperatures[*pindexT] > 0) { // usual case
+    if (pgd->rgInvTemperatures[*pindexT] > 0) { /* usual case */
 
       /* first scale temporarily the kernelSD according to the temperature */
 
@@ -2353,7 +2341,7 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
       if (pMCVar->dKernelSD > pMCVar->dMaxKernelSD)
         pMCVar->dKernelSD = pMCVar->dMaxKernelSD;
 
-      // compute the integral of the jump kernel
+      /* compute the integral of the jump kernel */
       dLnKern  = log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) /
                                 pMCVar->dKernelSD) -
                      CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) /
@@ -2362,19 +2350,19 @@ void SampleThetasTempered (PLEVEL plevel, char **args)
       /* sample a new value */
       pMCVar->dVal = SampleTheta (pMCVar);
 
-      // update the integral of the jump kernel
+      /* update the integral of the jump kernel */
       dLnKernNew  =  log(CDFNormal ((MaxMCVar(pMCVar) - pMCVar->dVal) /
                                     pMCVar->dKernelSD) -
                          CDFNormal ((MinMCVar(pMCVar) - pMCVar->dVal) /
                                     pMCVar->dKernelSD));
     }
-    else { // inverse temperature is zero, sample uniformly or from the prior
+    else { /* inverse temperature is zero, sample uniformly or from the prior */
       dLnKern = dLnKernNew  = 1;
-      if (pgd->nSimTypeFlag == 3) { // the posterior is tempered
-        // sample a new value uniformly in its range
+      if (pgd->nSimTypeFlag == 3) { /* the posterior is tempered */
+        /* sample a new value uniformly in its range */
         pMCVar->dVal = SampleThetaUnif (pMCVar);
       }
-      else { // the likelihood is tempered, sample from the prior
+      else { /* the likelihood is tempered, sample from the prior */
         CalculateOneMCParm (pMCVar);
       }
     }
@@ -2426,14 +2414,12 @@ WriteIt: /* Write the MC var value to output file  */
 } /* SampleThetasTempered */
 
 
-
 /* Function -------------------------------------------------------------------
    SampleThetaVector
 
    Sample thetas in block, do Metropolis test, write the sampled values to
    the output file
 */
-
 void SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
                         double *pdTheta, double *pdSum, double **prgdSumProd,
                         long iter, long nUpdateAt, long nTotal,
@@ -2529,12 +2515,12 @@ void SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
 
   /* Set the dVals of the variables to the values sampled and check that
      we are within bounds */
-  long iTmp = 0; // dummy variable to avoid resetting nThetas FB 16/07/2016
+  long iTmp = 0; /* dummy variable to avoid resetting nThetas FB 16/07/2016 */
   bInBounds = TraverseLevels1 (pLevel, SetMCVars, pdTheta, &iTmp, NULL);
 
-  if (!bInBounds) { // reject
+  if (!bInBounds) { /* reject */
     for (i = 0; i < nThetas; i++)
-      pdTheta[i] = pdTheta_old[i]; // restore
+      pdTheta[i] = pdTheta_old[i]; /* restore */
     goto DontJump;
   }
 
@@ -2575,7 +2561,6 @@ void SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
       prgdSumProd[i][j] = prgdSumProd[i][j] + pdTheta[i] * pdTheta[j];
   }
 
-
 } /* SampleThetaVector */
 
 
@@ -2584,7 +2569,6 @@ void SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
 
    Called from TraverseLevels1
 */
-
 int SaveLikelihoods (PLEVEL plevel, char **args)
 {
   PEXPERIMENT pExpt = plevel->pexpt;
@@ -2603,7 +2587,6 @@ int SaveLikelihoods (PLEVEL plevel, char **args)
 
    Set the array of fixed variables
 */
-
 void SetFixedVars (PLEVEL plevel)
 {
   long n;
@@ -2625,7 +2608,6 @@ void SetFixedVars (PLEVEL plevel)
 
    Set initial values of the MCMC jumping kernel
 */
-
 void SetKernel (PLEVEL plevel, char **args)
 {
   intptr_t useMCVarVals = (intptr_t) args[0]; /* 1 to restore dVal, else 2 */
@@ -2688,7 +2670,6 @@ void SetKernel (PLEVEL plevel, char **args)
 
    Write values of the MCMC jumping kernel
 */
-
 void WriteKernel (PLEVEL plevel, char **args)
 {
   FILE   *pfile = (FILE *) args[0];
@@ -2710,7 +2691,6 @@ void WriteKernel (PLEVEL plevel, char **args)
 
    Read values of the MCMC jumping kernel
 */
-
 void ReadKernel (PLEVEL plevel, char **args)
 {
   FILE   *pfile = (FILE *) args[0];
@@ -2739,7 +2719,6 @@ void ReadKernel (PLEVEL plevel, char **args)
    Sets the array of model variables to the sampled values. Does not set fixed
    variables. That has to be done by SetFixedVars.
 */
-
 void SetModelVars(PLEVEL plevel)
 {
   long n;
@@ -2762,7 +2741,6 @@ void SetModelVars(PLEVEL plevel)
    It also checks that the ranges are respected.
    Called from TraverseLevels.
 */
-
 int SetMCVars (PLEVEL plevel, char **args)
 {
   double *pdMCVarVals = (double *) args[0];
@@ -2806,7 +2784,6 @@ int SetMCVars (PLEVEL plevel, char **args)
    pdParms point to their own dParms.
    The pdVals and pdParms of the data and output arrays are also set.
 */
-
 void SetPointers (PLEVEL plevel, char **args)
 {
   long i, j, k;
@@ -2908,7 +2885,6 @@ void SetPointers (PLEVEL plevel, char **args)
 
    Called from TraverseLevels1
 */
-
 int SumAllExpts (PLEVEL plevel, char **args)
 {
   double      *pdLnData = (double*)args[0];
@@ -2927,7 +2903,6 @@ int SumAllExpts (PLEVEL plevel, char **args)
 
    Test prior, likelihood against random number between 0 and 1
 */
-
 BOOL TestImpRatio (PGIBBSDATA pgd,  BOOL bExptIsDep,
                    double dLnKern,  double dLnKernNew,
                    double dLnPrior, double dLnPriorNew,
@@ -2969,7 +2944,6 @@ BOOL TestImpRatio (PGIBBSDATA pgd,  BOOL bExptIsDep,
    Test prior, likelihood against a random number between 0 and 1 according to
    the temperature
 */
-
 BOOL TestImpRatioTemper (PGIBBSDATA pgd,  BOOL bExptIsDep,
                          double dLnKern,  double dLnKernNew,
                          double dLnPrior, double dLnPriorNew,
@@ -2983,12 +2957,12 @@ BOOL TestImpRatioTemper (PGIBBSDATA pgd,  BOOL bExptIsDep,
       dLnDataNew  == NULL_SUPPORT)
     return FALSE;
 
-  if (pgd->nSimTypeFlag == 3) { // posterior is tempered
+  if (pgd->nSimTypeFlag == 3) { /* posterior is tempered */
     dPjump = pgd->rgInvTemperatures[indexT] *
              (dLnPriorNew - dLnPrior + dLnLikeNew - dLnLike +
               dLnKern - dLnKernNew);
   }
-  else { // only the likelihood is tempered
+  else { /* only the likelihood is tempered */
     dPjump = dLnPriorNew - dLnPrior +
              pgd->rgInvTemperatures[indexT] *
              (dLnLikeNew - dLnLike + dLnKern - dLnKernNew);
@@ -3007,7 +2981,6 @@ BOOL TestImpRatioTemper (PGIBBSDATA pgd,  BOOL bExptIsDep,
 
    Test temperature against a random number between 0 and 1
 */
-
 BOOL TestTemper (PGIBBSDATA pgd, long indexT, long indexT_new, double dLnPrior,
                  double dLnData, double pseudo, double pseudonew)
 {
@@ -3054,7 +3027,6 @@ BOOL TestTemper (PGIBBSDATA pgd, long indexT, long indexT_new, double dLnPrior,
    The argument list is saved from the initial call; on recursive calls the
    list is NULL
 */
-
 void TraverseLevels (PLEVEL plevel,
                      void (*routinePtr)(PLEVEL plevel, char **args), ...)
 {
@@ -3117,7 +3089,6 @@ int TraverseLevels1 (PLEVEL plevel,
    Called from Traverse Levels
    Write the names of the sampled parameters to output file header
 */
-
 void WriteHeader (PLEVEL plevel, char **args)
 {
   PANALYSIS panal = (PANALYSIS)args[0];
@@ -3142,7 +3113,6 @@ void WriteHeader (PLEVEL plevel, char **args)
 
    Write the values of MC vars for one level to output file
 */
-
 void WriteMCVars (PLEVEL plevel, PFILE pOutFile)
 {
   long n;
@@ -3154,3 +3124,5 @@ void WriteMCVars (PLEVEL plevel, PFILE pOutFile)
   }
 
 } /* WriteMCVars */
+
+/* End */
