@@ -3,7 +3,7 @@
    Written by Don Maszle
    7 October 1991
 
-   Copyright (c) 1991-2008 Free Software Foundation, Inc.
+   Copyright (c) 1991-2017 Free Software Foundation, Inc.
 
    This file is part of GNU MCSim.
 
@@ -20,14 +20,6 @@
    You should have received a copy of the GNU General Public License
    along with GNU MCSim; if not, see <http://www.gnu.org/licenses/>
 
-   -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
-      Author:  @a
-   -- SCCS  ---------
-
    Model utilities.
 
    This file contains a number of utility routines used by the model
@@ -92,13 +84,7 @@ BOOL vbModelReinitd = FALSE;    /* Model has been initialized */
    handle to 'Decay' would be the hDecay parameter of that input
    function.
 
-   This routine also adjust variant exposure times, e.g. for
-   PerExp's, the exposure time is set to N_TAU_EXPOSE Tau's. At this
-   point, the input has decayed to a neglible quantity, and we don't
-   have to waste time calling the exp() function.
-
-   After N_TAU_EXPOSE == 40 Tau's, the input has decayed to 4.2e-18
-   of the magnitude.
+   This routine also adjust variant exposure times.
 */
 
 void FixupDependentInputs (void)
@@ -144,11 +130,9 @@ void FixupDependentInputs (void)
         }
       }
 
-    /* Fix exponential exposure times for efficiency */
+    /* set exponential exposure times */
     if ((vrgInputs[i].iType == IFN_PEREXP) ||
         (vrgInputs[i].iType == IFN_PERTRANS)) {
-      vrgInputs[i].dTexp = N_TAU_EXPOSE / vrgInputs[i].dDecay;
-      if (vrgInputs[i].dTper < vrgInputs[i].dTexp)
         vrgInputs[i].dTexp = vrgInputs[i].dTper;
     }
 
@@ -160,8 +144,6 @@ void FixupDependentInputs (void)
 		"in PerTransit input function - Exiting\n");
         exit (0);
       }
-      // vrgInputs[i].dTexp = N_TAU_EXPOSE / vrgInputs[i].dDecay;
-      // if (vrgInputs[i].dTper < vrgInputs[i].dTexp)
       vrgInputs[i].dTexp = vrgInputs[i].dTper;
     }
 
@@ -889,7 +871,7 @@ void GetStateHandles (HVAR *phvar)
 
   VMMAPSTRCT *pvm = vrgvmGlo;
 
-  while (pvm->szName) {
+  while (pvm->pVar) {
     if (IsState(pvm->hvar))
       phvar[i++] = pvm->hvar;
     ++pvm;
