@@ -1,6 +1,6 @@
 /* random.h
 
-   Copyright (c) 1992-2008 Free Software Foundation, Inc.
+   Copyright (c) 1992-2017 Free Software Foundation, Inc.
 
    This file is part of GNU MCSim.
 
@@ -17,19 +17,10 @@
    You should have received a copy of the GNU General Public License
    along with GNU MCSim; if not, see <http://www.gnu.org/licenses/>
 
-   -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
-      Author:  @a
-   -- SCCS  ---------
-
    Header for random number generator.  See random.c for extensive
    documentation.
 
    Gives prototypes for random functions.
-
 */
 
 #ifndef RANDOM_H_DEFINED
