@@ -1,6 +1,6 @@
 /* siminit.c
 
-   Copyright (c) 1991-2008 Free Software Foundation, Inc.
+   Copyright (c) 1991-2017 Free Software Foundation, Inc.
 
    This file is part of GNU MCSim.
 
@@ -17,14 +17,6 @@
    You should have received a copy of the GNU General Public License
    along with GNU MCSim; if not, see <http://www.gnu.org/licenses/>
 
-   -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
-      Author:  @a
-   -- SCCS  ---------
-
    Contains initialization routines for the simulation
 */
 
@@ -68,7 +60,7 @@ void InitIntegratorSpec (PINTSPEC pis)
   pis->itask = ITASK_DEFAULT;
   pis->itol  = ITOL_DEFAULT;
   pis->dRtol = RTOL_DEFAULT;
-  pis->dAtol = RTOL_DEFAULT;
+  pis->dAtol = ATOL_DEFAULT;
   pis->iMf   = IMF_DEFAULT;
   pis->liw   = LSODES_IWORKSIZE;
   pis->lrw   = LSODES_RWORKSIZE;
@@ -152,6 +144,9 @@ void InitMonteCarlo (PMONTECARLO pmc)
   pmc->pfileSetPoints = NULL;
 
   pmc->plistMCVars   = NULL;
+  pmc->rgdParms = NULL;
+  pmc->rghvar = NULL;
+  pmc->rgpMCVar = NULL;
 
 } /* InitMonteCarlo */
 
@@ -212,9 +207,10 @@ void InitAnalysis (PANALYSIS panal)
   /* Global experiment settings */
   InitExperiment (&panal->expGlobal, &panal->modelinfo);
 
-  panal->szOutfilename    = NULL;
-  panal->pfileOut         = NULL;
-  panal->bCommandLineSpec = FALSE;
+  panal->szOutfilename      = NULL;
+  panal->pfileOut           = NULL;
+  panal->bCommandLineSpec   = FALSE;
+  panal->bAllocatedFileName = FALSE;
 
   /* Init all experiments NULL */
   panal->iExpts = 0;
@@ -422,8 +418,6 @@ BOOL PrepareOutSpec (PEXPERIMENT pexp)
       ForAllList (pos->plistPrintRecs, InitOneOutVar, (PVOID) &pexp->os);
       pos->nOutputs = cOut; /* Set count of outputs for real */
 
-      FreeList (&pos->plistPrintRecs, NULL, TRUE);
-
       CreateOutputSchedule (pos); /* Create single output time list */
 
       /* Adjust the final time to match the last output time */
